QLabel {
    font-family: Microsoft YaHei;
}

QFrame#mFrame {
    background-color: white;
}

RepoTreeView {
    font-family: Microsoft YaHei;
}

FileTableView QHeaderView::section:first {
    padding-left: 36px;
}

SyncErrorsTableView::item {
    padding-left: 2px;
}
