// The class IDs of these Shell extension class.
//
// class ids:
//
// D14BEDD3-4E05-4F2F-B0DE-C0381E6AE606
//
//

DEFINE_GUID(CLSID_SEAFILE_SHELLEXT,0xd14bedd3, 0x4e05, 0x4f2f, 0xb0, 0xde, 0xc0, 0x38, 0x1e, 0x6a, 0xe6, 0x06);

DEFINE_GUID(CLSID_SEAFILE_ICON_NORMAL,0xd14bedd3, 0x4e05, 0x4f2f, 0xb0, 0xde, 0xc0, 0x38, 0x1e, 0x6a, 0xe6, 0x07);

DEFINE_GUID(CLSID_SEAFILE_ICON_SYNCING,0xd14bedd3, 0x4e05, 0x4f2f, 0xb0, 0xde, 0xc0, 0x38, 0x1e, 0x6a, 0xe6, 0x08);

DEFINE_GUID(CLSID_SEAFILE_ICON_ERROR,0xd14bedd3, 0x4e05, 0x4f2f, 0xb0, 0xde, 0xc0, 0x38, 0x1e, 0x6a, 0xe6, 0x09);

DEFINE_GUID(CLSID_SEAFILE_ICON_PAUSED,0xd14bedd3, 0x4e05, 0x4f2f, 0xb0, 0xde, 0xc0, 0x38, 0x1e, 0x6a, 0xe6, 0x10);

DEFINE_GUID(CLSID_SEAFILE_ICON_LOCKED_BY_ME,0xd14bedd3, 0x4e05, 0x4f2f, 0xb0, 0xde, 0xc0, 0x38, 0x1e, 0x6a, 0xe6, 0x11);

DEFINE_GUID(CLSID_SEAFILE_ICON_LOCKED_BY_OTHERS,0xd14bedd3, 0x4e05, 0x4f2f, 0xb0, 0xde, 0xc0, 0x38, 0x1e, 0x6a, 0xe6, 0x12);
