1 ICON  DISCARDABLE "paused.ico"
2 ICON  DISCARDABLE "normal.ico"
3 ICON  DISCARDABLE "syncing.ico"
4 ICON  DISCARDABLE "error.ico"
5 ICON  DISCARDABLE "locked-by-me.ico"
6 ICON  DISCARDABLE "locked-by-others.ico"
