#ifndef SEAFILE_CLIENT_FINDER_SYNC_LISTENER_H_
#define SEAFILE_CLIENT_FINDER_SYNC_LISTENER_H_

/// \brief start the listener
///
void finderSyncListenerStart();

/// \brief stop the listener
///
void finderSyncListenerStop();

#endif // SEAFILE_CLIENT_FINDER_SYNC_LISTENER_H_
