
#ifndef SEAFILE_CELINT_TRANSLATE_COMMIT_DESC_H
#define SEAFILE_CELINT_TRANSLATE_COMMIT_DESC_H

#include <QString>

QString
translateCommitDesc (const QString& input);

#endif  // SEAFILE_CELINT_TRANSLATE_COMMIT_DESC_H
